# EventGhost JRiver Media Center Event Plugin
# Copyright (C) 2014 Geoff Van Brunt
# 
# Description:  EventGhost JRiver Media Center Event Plugin 
#                        fires Events when tracks changed,
#                        tracks paused/stopped/played etc.
#
# Instructions: 1. Install the plugin under the EventGhost directory
#               	in plugins
#               2. Add the plug in in Eventghost. On the menu select
#               	configuration->Add Plugin. It is under the other category
#               3. Configure the plug in by adding you Access Key, Username,
#               	Password and the polling interval.It's best to use a long
#               	interval if possible to avoid hammering your MC.
#               4. Subscribe to the approprirate events when you see them fired.
#               	You can use a python script after the event to work with 
#               	work with the fields of the track they are stored in the 
#					eg.event.payload.
#

import eg

eg.RegisterPlugin(
	name = "JRiver Media Center Event Plugin",
	author = "Geoff Van Brunt",
	version = "0.0.1",
	king = "other",
	description = """This plugin polls JRiver Media Center for playback events 
		and fires Events when tracks changed, tracks paused/stopped/played etc.""")

import urllib2
import xml.etree.ElementTree as ET
from threading import Event, Thread


DEBUG = False
if DEBUG:
    log = eg.Print
else:
    def log(dummyMesg):
        pass

class MCEventPlugin(eg.PluginBase):
	mediaCenterUrl = ''
	token = ''
	storedInfoDict = {}
	pollingInterval = 0
	accessKey = ''
	userName = ''
	passWord = ''
	isInitialized = False


	def Configure(self, accessKey='', userName='', passWord='', pollingInterval=5):
		log("Configureing")
		panel = eg.ConfigPanel()
		accessKeyCtrl = panel.TextCtrl(accessKey)
		userNameCtrl = panel.TextCtrl(userName)
		passWordCtrl = panel.TextCtrl(passWord, style=wx.TE_PASSWORD)
		pollingIntervalCtrl = panel.SpinNumCtrl(pollingInterval)
		lbl1 = panel.StaticText("Access Key:")
		lbl2 = panel.StaticText("User Name:")
		lbl3 = panel.StaticText("Password:")
		lbl4 = panel.StaticText("Polling Interval (sec):")
		eg.EqualizeWidths((lbl1, lbl2, lbl3, lbl4))
		box1 = panel.BoxedGroup("Media Center Settings", (lbl1, accessKeyCtrl), (lbl2, userNameCtrl), (lbl3, passWordCtrl), (lbl4, pollingIntervalCtrl))
		panel.sizer.Add(box1, 0, wx.EXPAND)
		while panel.Affirmed():
			panel.SetResult(
			accessKeyCtrl.GetValue(),
			userNameCtrl.GetValue(),
			passWordCtrl.GetValue(),
			pollingIntervalCtrl.GetValue()
			)

	def __start__(self, accessKey, userName, passWord, pollingInterval):
		log("Starting up")
		self.accessKey = accessKey
		self.userName = userName
		self.passWord = passWord
		self.pollingInterval = pollingInterval

		self.pollThread = Event()
		thread = Thread(
			target = self.PollingLoop,
			args = (self.pollThread, )
		)
		thread.start()

	def __stop__(self):
		self.pollThread.set()

	def PollingLoop(self, pollThread):
		while not pollThread.isSet():
			log("Top of polling thread")
			if not self.isInitialized: # Initialize if not already
				self.Initialize()
			else:
				try:
					response = urllib2.urlopen("{0}/Playback/Info?Token={1}".format(self.mediaCenterUrl,self.token))
					log("Got Playback/Info")
					playInfoDict = self.GetItems(response)

					response = urllib2.urlopen("{0}/File/GetInfo?File={1}&Token={2}".format(self.mediaCenterUrl,playInfoDict['FileKey'],self.token))
					log("Got File/GetInfo")
					fileInfoDict = self.ParseMPL(response)
					combinedInfoDict = dict(playInfoDict.items() + fileInfoDict.items())

					if self.storedInfoDict == {}:
						self.storedInfoDict = combinedInfoDict

					if self.storedInfoDict['FileKey'] != playInfoDict['FileKey']:
						self.TriggerEvent("TrackChanged", combinedInfoDict)

					if self.storedInfoDict['State'] != combinedInfoDict['State']:
						if int(combinedInfoDict['State']) == 0:
							self.TriggerEvent('Stop', combinedInfoDict)
						elif int(combinedInfoDict['State']) == 1:
							self.TriggerEvent('Pause', combinedInfoDict)
						elif int(combinedInfoDict['State']) == 2:
							self.TriggerEvent('Play', combinedInfoDict)

					self.storedInfoDict = combinedInfoDict
				except urllib2.HTTPError as e:
					if 	e.code == 401: # Make sure we have a working authenticaiton token
						self.GetAuthToken()
					else:
						log('MCEventPlugin Exception in PollingLoop: {0}'.format(e))
				except Exception as e:
					log('MCEventPlugin Exception in PollingLoop: {0}'.format(e))

			pollThread.wait(self.pollingInterval)

	def Initialize(self):
		try:
			log("Initializing")

			# Get IP Address and Port from Access Key
			response = urllib2.urlopen("http://webplay.jriver.com/libraryserver/lookup?id={0}".format(self.accessKey))
			responseString = response.read()
			ipAddress = responseString[responseString.find("<ip>")+4:responseString.find("</ip>")]
			port = responseString[responseString.find("<port>")+6:responseString.find("</port>")]

			self.mediaCenterUrl = "http://{0}:{1}/MCWS/v1".format(ipAddress,port)

			self.isInitialized = True
		except Exception as e:
			log('MCEventPlugin Exception during initialization: {0}'.format(e))

	def GetAuthToken(self):
		try:
			log("Getting Authentication Token")

			# Get Authentication Token
			passman = urllib2.HTTPPasswordMgrWithDefaultRealm()
			passman.add_password(None, self.mediaCenterUrl, self.userName, self.passWord)
			authhandler = urllib2.HTTPBasicAuthHandler(passman)
			opener = urllib2.build_opener(authhandler)
			urllib2.install_opener(opener)
			response = urllib2.urlopen(self.mediaCenterUrl + "/Authenticate")

			self.token = self.GetElement(response, 'Token')
		except Exception as e:
			print 'MCEventPlugin Exception getting authentication token: {0}'.format(e)

	def GetElement(self, response, itemName):
		tree = ET.parse(response)
		lst = tree.findall('Item')
		for item in lst:
			if item.get('Name') == itemName:
				return item.text
		return None

	def GetItems(self, response):
		dictionary = {}
		tree = ET.parse(response)
		itemList = tree.findall('Item')
		for item in itemList:
			dictionary[item.get('Name')] = item.text
		return dictionary

	def ParseMPL(self, response):
		dictionary = {}
		tree = ET.parse(response)
		itemList = tree.findall('/Item/Field')
		for item in itemList:
			dictionary[item.get('Name')] = item.text
		return dictionary